@@ -8,7 +8,10 @@ use Carp ();
 use Module::Load ();
 use Furl::HTTP;
 
-our $VERSION = '0.02';
+our $VERSION = '0.10';
+
+my $PM_PATH = $INC{ join( '/', split('::', __PACKAGE__) ) . '.pm' };
+$PM_PATH =~ s/\.pm$//;
 
 my @SUPPORTED_API    = _plugin_list('API');
 my @SUPPORTED_DRIVER = _plugin_list('Driver');
@@ -17,13 +20,14 @@ my @SUPPORTED_DRIVER = _plugin_list('Driver');
 sub _options
 {
     return {
+        idf_type      => 1,
         documents     => 250_0000_0000,
         df_file       => undef,
         fetch_df      => 1,
         default_df    => 5000,
         expires_in    => 365, # number of days
         driver        => 'Storable',
-        api           => 'Bing',
+        api           => 'Yahoo',
         appid         => undef,
         Furl_HTTP     => undef,
     };
@@ -39,12 +43,13 @@ sub new
     for my $key (keys %args)
     {
         if (!exists $options->{$key}) { Carp::croak("Unknown option: $key"); }
-        else                          { $options->{$key} = $args{$key}; }
+        else                          { $options->{$key} = $args{$key};      }
     }
 
-    Carp::croak('appid is needed')                    unless defined $options->{appid};
-    Carp::croak("Unknown driver: $options->{driver}") unless grep { $options->{driver} eq $_ } @SUPPORTED_DRIVER;
-    Carp::croak("Unknown api: $options->{api}")       unless grep { $options->{api}    eq $_ } @SUPPORTED_API;
+    Carp::croak('appid is needed')                        unless defined $options->{appid};
+    Carp::croak("Unknown driver: $options->{driver}")     unless grep { $options->{driver} eq $_   } @SUPPORTED_DRIVER;
+    Carp::croak("Unknown api: $options->{api}")           unless grep { $options->{api}    eq $_   } @SUPPORTED_API;
+    Carp::croak("Unknown idf type: $options->{idf_type}") unless grep { $options->{idf_type} eq $_ } 1 .. 3;
 
     if (defined $options->{Furl_HTTP})
     {
@@ -55,14 +60,7 @@ sub new
     Module::Load::load(__PACKAGE__ . '::API::' . $options->{api});
     Module::Load::load(__PACKAGE__ . '::Driver::' . $options->{driver});
 
-    if (!defined $options->{df_file})
-    {
-        my $path = $INC{ join( '/', split('::', __PACKAGE__) ) . '.pm' };
-        $path =~ s/\.pm$//;
-        $path .= '/bing_utf8.st';
-
-        $options->{df_file} = $path;
-    }
+    $options->{df_file} = "$PM_PATH/yahoo_utf8.st" unless defined $options->{df_file};
 
     bless $options, $class;
 }
@@ -73,16 +71,26 @@ sub idf
 
     if (!defined $word)
     {
-        Carp::carp("Undefined word was set to idf method");
+        Carp::carp("Undefined word has been set to idf method");
         return;
     }
 
-    my $df = $self->df($word);
-    my $N  = $self->{documents};
+    my $df   = $self->df($word);
+    my $N    = $self->{documents};
+    my $type = $self->{idf_type};
 
-    $df = 1 if $df == 0; # To avoid dividing by zero
+    my $idf;
 
-    return log($N / $df);
+    if ($type == 1)
+    {
+        $df = 1 if $df == 0; # To avoid dividing by zero
+        $idf = log($N / $df);
+    }
+    elsif ($type == 2) { $idf = log( ($N - $df + 0.5) / ($df + 0.5) ); }
+    elsif ($type == 3) { $idf = log( ($N + 0.5) / ($df + 0.5) ); }
+    else               { Crap::croak("Unknown idf_type: $type"); }
+
+    return $idf;
 }
 
 sub df
@@ -91,7 +99,7 @@ sub df
 
     if (!defined $word)
     {
-        Carp::carp("Undefined word was set to df method");
+        Carp::carp("Undefined word has been set to df method");
         return;
     }
 
@@ -148,17 +156,14 @@ sub _fetch_new_df
 
     no strict 'refs';
     my $api = __PACKAGE__ . '::API::' . $self->{api};
-    &{$api . '::fetch_new_df'}($self, $word);
+    &{$api . '::fetch_new_df'}($word, $self->{furl_http}, $self->{appid});
 }
 
 sub _plugin_list
 {
     my $type = shift;
 
-    my $path = $INC{ join( '/', split('::', __PACKAGE__) ) . '.pm' };
-    $path =~ s/\.pm$//;
-
-    my $dir = "$path/$type/";
+    my $dir = "$PM_PATH/$type/";
 
     opendir(my $dh, $dir) or Carp::croak("Can't open $dir: $!");
     my @contents = readdir $dh;
@@ -188,7 +193,7 @@ my ($appid);
 
   my $webidf = Lingua::JA::WebIDF->new
   (
-      api       => 'Bing',
+      api       => 'Yahoo',
       appid     => $appid,
       fetch_df  => 1,
       Furl_HTTP => { timeout => 3 }
@@ -205,6 +210,10 @@ WebIDF(Inverse Document Frequency) scores represent the rarity of words on the W
 The WebIDF scores of rare words are high.
 Conversely, the WebIDF scores of common words are low.
 
+IDF is based on the intuition that a query term which occurs in
+many documents is not a good discriminator and should be given less weight
+than one which occurs in few documents.
+
 =head1 METHOD
 
 =head2 new( %config || \%config )
@@ -215,7 +224,8 @@ The following configuration is used if you don't set %config.
 
   KEY                 DEFAULT VALUE
   -----------         ---------------
-  api                 'Bing'
+  idf_type            1
+  api                 'Yahoo'
   appid               undef
   driver              'Storable'
   df_file             undef
@@ -227,7 +237,33 @@ The following configuration is used if you don't set %config.
 
 =over 4
 
-=item api => 'Bing' || 'Yahoo' || 'YahooPremium'
+=item idf_type => 1 || 2 || 3
+
+The type1 is the most commonly cited form of IDF.
+
+                   N
+  idf(t_i) = log -----  (1)
+                  n_i
+
+  N  : the number of documents
+  n_i: the number of documents which contain term t_i
+  t_i: term
+
+
+The type2 is a simple version of the RSJ weight.
+
+              N - n_i + 0.5
+  w_i = log ----------------  (2)
+               n_i + 0.5
+
+
+The type3 is a modification of (2).
+
+              N + 0.5
+  w_i = log -----------  (3)
+             n_i + 0.5
+
+=item api => 'Yahoo' || 'YahooPremium' || 'Bing'
 
 Uses the specified Web API when fetches WebDF(Document Frequency) scores
 from the Web.
@@ -240,7 +276,7 @@ Fetches and saves WebDF scores with the specified driver.
 
 Saves WebDF scores to the specified path.
 
-If undef is specified, 'bing_utf8.st' is used.
+If undef is specified, 'yahoo_utf8.st' is used.
 This file is located in 'Lingua/JA/WebIDF/'
 and contains the WebDF scores of about 60000 words.
 There are other format files in the 'df' directory of this library.
@@ -289,11 +325,15 @@ pawa E<lt>pawapawa@cpan.orgE<gt>
 
 L<Lingua::JA::TFIDF>
 
-L<http://www.bing.com/toolbox/bingdeveloper/>
+Bing API: L<http://www.bing.com/toolbox/bingdeveloper/>
 
-L<http://developer.yahoo.co.jp/>
+Yahoo API: L<http://developer.yahoo.co.jp/>
 
-L<http://fallabs.com/tokyocabinet/>
+Tokyo Cabinet: L<http://fallabs.com/tokyocabinet/>
+
+S. Robertson, Understanding inverse document frequency:
+on theoretical arguments for IDF.
+Journal of Documentation 60, 503-520, 2004.
 
 =head1 LICENSE
 
